-- SLIDE 2/9 -------------------------------------------------------------------
CREATE TABLE Student (
  login CHAR(6) PRIMARY KEY,
  fname VARCHAR(30) NOT NULL,
  lname VARCHAR(50) NOT NULL,
  email VARCHAR(50) NOT NULL,
  tallness INT NOT NULL);

-- SLIDE 3/9 -------------------------------------------------------------------
ALTER TABLE STUDENT ADD isTall INT NOT NULL CHECK (isTall IN (0, 1));

INSERT INTO Student (login, fname, lname, email, tallness, isTall) 
VALUES ('luk194', 'Petr', 'Lukas', 'petr.lukas@vsb.cz', 183, 0);
INSERT INTO Student (login, fname, lname, email, tallness, isTall)
VALUES ('nie074', 'Lukas', 'Niederle', 'lukas.niederle@vsb.cz', 175, 0);
INSERT INTO Student (login, fname, lname, email, tallness, isTall)
VALUES ('hof078', 'Jiri', 'Hofrichter', 'jiri.hofrichter@vsb.cz', 168, 0);

CREATE OR REPLACE PROCEDURE IsStudentTall(p_login CHAR) AS
  v_avgTallness FLOAT;
  v_tallness Student.tallness%TYPE;
BEGIN
  SELECT AVG(tallness) INTO v_avgTallness FROM Student;
  SELECT tallness INTO v_tallness FROM Student WHERE login = p_login;
  
  IF v_tallness < v_avgTallness THEN
    UPDATE Student SET isTall = 0 WHERE login = p_login;
  ELSE
    UPDATE Student SET isTall = 1 WHERE login = p_login;
  END IF;
END;

CREATE OR REPLACE FUNCTION LoginExists(p_login CHAR) RETURN BOOLEAN AS
  v_login Student.login%TYPE;
BEGIN
  SELECT login INTO v_login FROM STUDENT WHERE login = p_login;
  RETURN TRUE;
EXCEPTION
  WHEN OTHERS THEN
    RETURN FALSE;
END;

ALTER TABLE Student MODIFY email VARCHAR(50) NULL;

CREATE OR REPLACE PROCEDURE AddStudent2(p_fname VARCHAR, p_lname VARCHAR,
  p_tallness INT) AS
  v_login Student.login%TYPE;
  v_counter INT;
BEGIN
  v_counter := 1;
  LOOP
    v_login := LOWER(SUBSTR(p_lname, 0, 3)) || 
      SUBSTR('000' || v_counter, LENGTH('000' || v_counter)-2, 3);
    EXIT WHEN NOT LoginExists(v_login);
    v_counter := v_counter + 1;
  END LOOP;
  INSERT INTO Student (login, fname, lname, tallness, istall)
  VALUES (v_login, p_fname, p_lname, p_tallness, 0);
END;

EXECUTE AddStudent2('Petr', 'Lukas', 183);
SELECT * FROM Student;

-- SLIDE 4/9 -------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE IsStudentTall AS
  CURSOR c_student IS SELECT * FROM Student;
  v_s Student%ROWTYPE;
  v_avgTallness FLOAT;
BEGIN
  SELECT AVG(tallness) INTO v_avgTallness FROM Student;
  
  OPEN c_student;
  LOOP
    FETCH c_student INTO v_s;
    EXIT WHEN c_student%NOTFOUND;
  
    IF (v_s.tallness < v_avgTallness) THEN
      UPDATE Student SET istall = 0 WHERE login=v_s.login;
    ELSE
      UPDATE Student SET istall = 1 WHERE login=v_s.login;
    END IF;
  END LOOP;
  
  CLOSE c_student;
END;

EXECUTE IsStudentTall;
SELECT * FROM Student;

CREATE OR REPLACE PROCEDURE IsStudentTall AS
  v_avgTallness FLOAT;
BEGIN
  SELECT AVG(tallness) INTO v_avgTallness FROM Student;
  
  FOR v_s IN (SELECT * FROM Student) LOOP
    IF (v_s.tallness < v_avgTallness) THEN
      UPDATE Student SET istall = 0 WHERE login=v_s.login;
    ELSE
      UPDATE Student SET istall = 1 WHERE login=v_s.login;
    END IF;
  END LOOP;
END;

-- SLIDE 5/9 -------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE CopyTableStructure(p_table_name VARCHAR) AS
  v_sql VARCHAR(1000);
  v_pk_cols VARCHAR(1000);
BEGIN
  FOR v_col IN (SELECT * FROM user_tab_columns WHERE table_name = p_table_name) LOOP
    v_sql := v_sql || CHR(13) || CHR(10) || '  ';

    v_sql := v_sql || v_col.column_name || ' ' ||
      v_col.data_type || '(' || v_col.data_length;
    IF v_col.data_scale IS NOT NULL THEN
      v_sql := v_sql || ', ' || v_col.data_scale;
    END IF;
    v_sql := v_sql || ') ';
    
    IF v_col.nullable = 'Y' THEN
      v_sql := v_sql || 'NULL';
    ELSE
      v_sql := v_sql || 'NOT NULL';
    END IF;

    v_sql := v_sql || ',';
  END LOOP;
  
  -- Primary key cols
  FOR v_primary_col IN (
    SELECT
      column_name
    FROM
      user_cons_columns
      JOIN user_constraints ON user_constraints.constraint_name = user_cons_columns.constraint_name
    WHERE
      user_cons_columns.TABLE_NAME = 'STUDENT' AND
      user_constraints.constraint_type = 'P') LOOP
    IF v_pk_cols IS NOT NULL THEN
      v_pk_cols := v_pk_cols || ', ';
    END IF;
    v_pk_cols := v_pk_cols || v_primary_col.column_name;
  END LOOP;
  v_sql := v_sql || CHR(13) || CHR(10) || '  ';
  v_sql := v_sql || 'PRIMARY KEY (' || v_pk_cols || ')';
  
  v_sql := 'CREATE TABLE old_' || p_table_name || '(' || v_sql ||
    CHR(13) || CHR(10) || ')';

  EXECUTE IMMEDIATE v_sql;  
END;

CREATE OR REPLACE PROCEDURE CopyTable(p_table_name VARCHAR) AS
  v_new_table_name VARCHAR(100);
  v_insert_sql VARCHAR(1000);
  v_insert_columns VARCHAR(1000);
  v_insert_values VARCHAR(1000);
BEGIN
  v_new_table_name := 'old_' || p_table_name;
  CopyTableStructure(p_table_name);

  EXECUTE IMMEDIATE 'INSERT INTO ' || v_new_table_name || ' ' ||
    'SELECT * FROM ' || p_table_name;
END;


-- SLIDE 6/9 -------------------------------------------------------------------

CREATE OR REPLACE FUNCTION LoginExists(p_login CHAR) RETURN BOOLEAN AS
  v_login Student.login%TYPE;
BEGIN
  SELECT login INTO v_login FROM STUDENT WHERE login = p_login;
  RETURN TRUE;
EXCEPTION
  WHEN NO_DATA_FOUND THEN
    RETURN FALSE;
END;

CREATE OR REPLACE FUNCTION InsertStudent(p_login CHAR, p_fname VARCHAR,
  p_lname VARCHAR, p_tallness INT) RETURN BOOLEAN AS
BEGIN
  INSERT INTO student (login, fname, lname, tallness, istall)
  VALUES (p_login, p_fname, p_lname, p_tallness, 0);
  RETURN TRUE;
EXCEPTION
  WHEN DUP_VAL_ON_INDEX THEN
    RETURN FALSE;
END;

CREATE OR REPLACE PROCEDURE AddStudent3(p_fname VARCHAR, p_lname VARCHAR,
  p_tallness INT) AS
  v_login Student.login%TYPE;
  v_counter INT;
BEGIN
  v_counter := 1;
  LOOP
    v_login := LOWER(SUBSTR(p_lname, 0, 3)) ||
      SUBSTR('000' || v_counter, LENGTH('000' || v_counter)-2, 3);
    EXIT WHEN InsertStudent(v_login, p_fname, p_lname, p_tallness);
    v_counter := v_counter + 1;
  END LOOP;
END;

EXECUTE AddStudent3('Petr', 'Lukas', 183);

-- SLIDE 7/9 -------------------------------------------------------------------
-- toto, at si udelaji doma... zadani je nepresne a pracuje s nenadefinovanymi
-- tabulkami. Pro ukazku, jak se daji vytvaret vlastni vyjimky.
DECLARE
  ex EXCEPTION;
BEGIN
  dbms_output.put_line('A');
  RAISE ex;
  dbms_output.put_line('B');
EXCEPTION
  WHEN ex THEN
    dbms_output.put_line('exception');
END;


-- SLIDE 8/9 -------------------------------------------------------------------

CREATE TABLE Statistics
(
  operation VARCHAR(10) PRIMARY KEY,
  operationCount INT NOT NULL
);
INSERT INTO Statistics VALUES ('INSERT', 0);
INSERT INTO Statistics VALUES ('UPDATE', 0);
INSERT INTO Statistics VALUES ('DELETE', 0);

CREATE OR REPLACE TRIGGER OperationCount BEFORE INSERT OR UPDATE OR DELETE ON student
BEGIN
  IF INSERTING THEN
    UPDATE Statistics
    SET operationCount = operationCount + 1
    WHERE operation = 'INSERT';
  ELSIF DELETING THEN
    UPDATE Statistics
    SET operationCount = operationCount + 1
    WHERE operation = 'DELETE';
  ELSIF UPDATING THEN
    UPDATE Statistics
    SET operationCount = operationCount + 1
    WHERE operation = 'UPDATE';
  END IF;
END;

CREATE TABLE COURSE
(
  code CHAR(11) PRIMARY KEY,
  name VARCHAR(50),
  capacity INT
);

CREATE TABLE STUDENT_COURSE
(
  login CHAR(6),
  code CHAR(11),
  PRIMARY KEY (login, code)
);

CREATE OR REPLACE TRIGGER CheckCapacity BEFORE INSERT OR UPDATE ON student_course FOR EACH ROW
DECLARE
  v_capacity INT;
  v_count INT;
BEGIN
  SELECT capacity INTO v_capacity FROM course WHERE code = :new.code;
  SELECT COUNT(*) INTO v_count FROM student_course WHERE code = :new.code;
  
  IF v_count + 1 > v_capacity THEN
    dbms_output.put_line('prekrocena kapacita');
  END IF;
END;

INSERT INTO course (code, name, capacity) VALUES ('12345678910', 'Test1', 2);
INSERT INTO course (code, name, capacity) VALUES ('12345678911', 'Test2', 3);
DELETE FROM student_course;
INSERT INTO student_course (login, code) VALUES ('luk002', '12345678910');
INSERT INTO student_course (login, code) VALUES ('luk003', '12345678910');
INSERT INTO student_course (login, code) VALUES ('luk004', '12345678910');
INSERT INTO student_course (login, code) VALUES ('luk005', '12345678910');

CREATE OR REPLACE TRIGGER CheckCapacity BEFORE INSERT OR UPDATE ON student_course FOR EACH ROW
DECLARE
  v_capacity INT;
  v_count INT;
  capacity_exceeded EXCEPTION;
BEGIN
  SELECT capacity INTO v_capacity FROM course WHERE code = :new.code;
  SELECT COUNT(*) INTO v_count FROM student_course WHERE code = :new.code;
  
  IF v_count + 1 > v_capacity THEN
    RAISE capacity_exceeded;
  END IF;
END;

-- SLIDE 9/9 -------------------------------------------------------------------

CREATE OR REPLACE PACKAGE student_package AS

capacity_exceeded EXCEPTION;
FUNCTION InsertStudent(p_login CHAR, p_fname VARCHAR, p_lname VARCHAR, p_tallness INT) RETURN BOOLEAN;
PROCEDURE AddStudent3(p_fname VARCHAR, p_lname VARCHAR, p_tallness INT);

END;

CREATE OR REPLACE PACKAGE BODY student_package AS
  FUNCTION InsertStudent(p_login CHAR, p_fname VARCHAR,
    p_lname VARCHAR, p_tallness INT) RETURN BOOLEAN AS
  BEGIN
    INSERT INTO student (login, fname, lname, tallness, istall)
    VALUES (p_login, p_fname, p_lname, p_tallness, 0);
    RETURN TRUE;
  EXCEPTION
    WHEN DUP_VAL_ON_INDEX THEN
      RETURN FALSE;
  END;
  
  PROCEDURE AddStudent3(p_fname VARCHAR, p_lname VARCHAR, p_tallness INT) AS
    v_login Student.login%TYPE;
    v_counter INT;
  BEGIN
    v_counter := 1;
    LOOP
      v_login := LOWER(SUBSTR(p_lname, 0, 3)) ||
        SUBSTR('000' || v_counter, LENGTH('000' || v_counter)-2, 3);
      EXIT WHEN InsertStudent(v_login, p_fname, p_lname, p_tallness);
      v_counter := v_counter + 1;
    END LOOP;
  END;
END;

EXECUTE student_package.AddStudent3('Jan', 'Vselijan', 123);