-- SLIDE 3/11 ----------------------------------------------------------------------------------------------------

CREATE PROCEDURE spAddStudent @p_login CHAR(6), @p_fname VARCHAR(30), @p_lname VARCHAR(50), @p_email VARCHAR(50) AS
BEGIN
	INSERT INTO Student(login, fname, lname, email) VALUES (@p_login, @p_fname, @p_lname, @p_email);
END;

EXEC spAddStudent 'luk184', 'Petr', 'Lukas', 'petr.lukas@vsb.cz'

CREATE PROCEDURE spAddStudent2 @p_login CHAR(6), @p_fname VARCHAR(30), @p_lname VARCHAR(50), @p_email VARCHAR(50), @p_result BIT OUTPUT AS
BEGIN
	BEGIN TRY
		INSERT INTO Student(login, fname, lname, email) VALUES (@p_login, @p_fname, @p_lname, @p_email);
		SET @p_result = 1;
	END TRY
	BEGIN CATCH
		SET @p_result = 0;
	END CATCH;
END;

SELECT 
    ERROR_NUMBER() AS ErrorNumber
    ,ERROR_SEVERITY() AS ErrorSeverity
    ,ERROR_STATE() AS ErrorState
    ,ERROR_LINE () AS ErrorLine
    ,ERROR_PROCEDURE() AS ErrorProcedure
    ,ERROR_MESSAGE() AS ErrorMessage;

BEGIN
	DECLARE @result BIT;
	EXEC spAddStudent2 'luk194', 'Petr', 'Lukas', 'petr.lukas@vsb.cz', @result OUTPUT
	IF @result = 1
		PRINT 'ok';
	ELSE
		PRINT 'nok';
END;

-- SLIDE 5/11 ----------------------------------------------------------------------------------------------------
CREATE PROCEDURE spStudentBecomeTeacher @p_login CHAR(6), @p_department INT AS
BEGIN
	INSERT INTO Teacher (login, fname, lname, email, department)
	SELECT login, fname, lname, email, @p_department
	FROM Student
	WHERE login = @p_login;

	DELETE FROM Student
	WHERE login = @p_login;
END;

EXEC spStudentBecomeTeacher 'luk194', 1


CREATE PROCEDURE spStudentBecomeTeacher2 @p_login CHAR(6), @p_department INT AS
BEGIN
	BEGIN TRANSACTION
	BEGIN TRY
		INSERT INTO Teacher (login, fname, lname, email, department)
		SELECT login, fname, lname, email, @p_department
		FROM Student
		WHERE login = @p_login;

		DELETE FROM Student
		WHERE login = @p_login;
		COMMIT;
	END TRY
	BEGIN CATCH
		ROLLBACK;
	END CATCH;
END;


-- SLIDE 6/11 ----------------------------------------------------------------------------------------------------
ALTER TABLE Student
ADD tallness INT NOT NULL DEFAULT 0;

-- SLIDE 7/11 ----------------------------------------------------------------------------------------------------
CREATE PROCEDURE spAddStudent3 @p_fname VARCHAR(30), @p_lname VARCHAR(50), @p_tallness INT AS
BEGIN
	DECLARE @v_login CHAR(6);
	DECLARE @v_email VARCHAR(50);
	SET @v_login = SUBSTRING(@p_lname, 1, 3) + '000';
	SET @v_email = @v_login + '@vsb.cz';
	INSERT INTO Student (login, fname, lname, email, tallness) VALUES (@v_login, @p_fname, @p_lname, @v_email, @p_tallness);
END;
	
EXEC spAddStudent3 'petr', 'lukas',	183

-- SLIDE 8/11 ----------------------------------------------------------------------------------------------------
ALTER TABLE Student
ADD isTall BIT NOT NULL DEFAULT 0;

CREATE PROCEDURE spIsStudentTall @p_login CHAR(6) AS
BEGIN
	DECLARE @v_avgTallness FLOAT;
	DECLARE @v_tallness INT;
	
	SET @v_avgTallness = (SELECT AVG(tallness) FROM Student);
	SET @v_tallness = (SELECT tallness FROM Student WHERE login = @p_login);
	
	IF @v_tallness >= @v_avgTallness
		UPDATE Student
		SET isTall = 1
		WHERE login = @p_login;
	ELSE
		UPDATE Student
		SET isTall = 0
		WHERE login = @p_login;
END;

CREATE FUNCTION sfLoginExists(@p_login CHAR(6)) RETURNS BIT AS
BEGIN
	DECLARE @result BIT;
	IF @p_login IN (SELECT login FROM Student)
		SET @result = 1;
	ELSE
		SET @result = 0;
	RETURN @result;
END;

PRINT dbo.sfLoginExists('luk195');

ALTER PROCEDURE spAddStudent4 @p_fname VARCHAR(30), @p_lname VARCHAR(50), @p_tallness INT AS
BEGIN
	DECLARE @v_login CHAR(6);
	DECLARE @v_email VARCHAR(50);
	DECLARE @v_counter INT;
	
	SET @v_counter = 0;
	
	SET @v_login = SUBSTRING(@p_lname, 1, 3) + RIGHT('000' + CAST(@v_counter AS VARCHAR), 3);
	WHILE (dbo.sfLoginExists(@v_login) = 1)
	BEGIN
		SET @v_login = SUBSTRING(@p_lname, 1, 3) + RIGHT('000' + CAST(@v_counter AS VARCHAR), 3);
		SET @v_counter = @v_counter + 1;
	END;
	
	SET @v_email = @v_login + '@vsb.cz';
	INSERT INTO Student (login, fname, lname, email, tallness) VALUES (@v_login, @p_fname, @p_lname, @v_email, @p_tallness);
END;
	
EXEC spAddStudent4 'Petr', 'Lukas', 183;


-- SLIDE 9/11 ----------------------------------------------------------------------------------------------------
ALTER PROCEDURE spSetStudentTallness AS 
BEGIN;
	DECLARE @v_login CHAR(6);
	DECLARE c CURSOR FOR SELECT login FROM Student;
	OPEN c;
	FETCH FROM c INTO @v_login;
	WHILE @@FETCH_STATUS = 0
	BEGIN
		EXEC spIsStudentTall @v_login
		FETCH FROM c INTO @v_login;
	END;
	
	CLOSE c;
	DEALLOCATE c;
END;
	
EXEC spSetStudentTallness;


-- SLIDE 10/11 ---------------------------------------------------------------------------------------------------
CREATE PROCEDURE spCopyTableStructure(@p_table_schema VARCHAR(30), @p_table_name VARCHAR(30)) AS
BEGIN
	DECLARE c_columnList CURSOR FOR 
		SELECT COLUMN_NAME, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH 
		FROM INFORMATION_SCHEMA.COLUMNS 
		WHERE TABLE_SCHEMA = @p_table_schema AND TABLE_NAME = @p_table_name;
	
	DECLARE @v_command NVARCHAR(1000);
	DECLARE @v_column_name VARCHAR(30);
	DECLARE @v_data_type VARCHAR(10);
	DECLARE @v_char_max_length INT;
	DECLARE @v_index INT;
	DECLARE @v_delim VARCHAR(1);

	OPEN c_columnList;
	FETCH NEXT FROM c_columnList INTO @v_column_name, @v_data_type, @v_char_max_length;

	SET @v_command = 'CREATE TABLE ' + @p_table_name + '_old (';
	SET @v_index = 0;

	WHILE @@FETCH_STATUS = 0
	BEGIN
		IF @v_index > 0
			SET @v_delim = ',';
		ELSE  
			SET @v_delim = '';

		SET @v_command = @v_command + @v_delim + @v_column_name + ' ' + @v_data_type;

		IF (@v_data_type = 'char' OR @v_data_type = 'varchar')
			SET @v_command = @v_command + '(' + CAST(@v_char_max_length AS VARCHAR(10)) + ')';
		  
		FETCH NEXT FROM c_columnList INTO @v_column_name, @v_data_type, @v_char_max_length;
		SET @v_index = @v_index + 1;
	END;

	SET @v_command = @v_command + ')';  
	print @v_command;

	EXEC sp_executesql @v_command;

	CLOSE c_columnList;
	DEALLOCATE c_columnList;  
END;

CREATE PROCEDURE spCopyTable(@p_table_schema VARCHAR(30), @p_table_name VARCHAR(30)) AS
BEGIN
  EXEC spCopyTableStructure @p_table_schema,@p_table_name;
  
  DECLARE @v_command NVARCHAR(1000);
  SET @v_command = 'INSERT INTO ' + @p_table_name + '_old SELECT * from ' + @p_table_name;
  EXEC sp_executesql @v_command;  
END;


-- SLIDE 11/11 ---------------------------------------------------------------------------------------------------

CREATE TABLE OpStatistics
(
	operation VARCHAR(10) PRIMARY KEY,
	cnt INT
);

INSERT INTO OpStatistics VALUES ('INSERT', 0);
INSERT INTO OpStatistics VALUES ('UPDATE', 0);
INSERT INTO OpStatistics VALUES ('DELETE', 0);

ALTER TRIGGER tgStatistics ON Student AFTER INSERT, UPDATE, DELETE AS
BEGIN
	DECLARE @v_inserted INT;
	DECLARE @v_deleted INT;
	
	SET @v_inserted = (SELECT COUNT(*) FROM inserted);
	SET @v_deleted = (SELECT COUNT(*) FROM deleted);

	IF @v_inserted > 0 AND @v_deleted = 0 
		UPDATE OpStatistics SET cnt = cnt + 1 WHERE operation = 'INSERT';
	ELSE IF @v_inserted > 0 AND @v_deleted > 0
		UPDATE OpStatistics SET cnt = cnt + 1 WHERE operation = 'UPDATE';
	ELSE IF @v_inserted = 0 AND @v_deleted > 0
		UPDATE OpStatistics SET cnt = cnt + 1 WHERE operation = 'DELETE';
END;

