-- Slide 1/9 --------------------------------------------------------------------

CREATE TABLE Teacher
(
	login CHAR(5) PRIMARY KEY,
	fname VARCHAR(30) NOT NULL,
	lname VARCHAR(50) NOT NULL,
	email VARCHAR(50) NOT NULL,
	department INT NOT NULL
);

CREATE TABLE Department
(
	id INT PRIMARY KEY IDENTITY (1, 1),
	name VARCHAR(50) NOT NULL,
	head CHAR(5) NULL
);

ALTER TABLE Teacher
ADD CONSTRAINT FK_Teacher_Department FOREIGN KEY (department) REFERENCES Department(id);

ALTER TABLE Department
ADD CONSTRAINT FK_Department_Head FOREIGN KEY (head) REFERENCES Teacher(login);

-- Slide 3/9 --------------------------------------------------------------------

ALTER TABLE Department DROP CONSTRAINT FK_Department_Head
DROP TABLE Teacher;
DROP TABLE Department;

-- Slide 5/9 --------------------------------------------------------------------

INSERT INTO Department (name) VALUES ('FEI');
INSERT INTO Department (name) VALUES ('FMMI');
INSERT INTO Department (name) VALUES ('HGF');
INSERT INTO Department (name) VALUES ('FBI');

INSERT INTO Teacher (login, fname, lname, email, department) VALUES ('luk06', 'Petr', 'Lukas', 'petr.lukas@vsb.cz', 1);
INSERT INTO Teacher (login, fname, lname, email, department) VALUES ('het01', 'James', 'Hetfied', 'james.hetfield@vsb.cz', 1);
INSERT INTO Teacher (login, fname, lname, email, department) VALUES ('toz01', 'Umberto', 'Tozzi', 'umberto.tozzi@vsb.cz', 2);
INSERT INTO Teacher (login, fname, lname, email, department) VALUES ('cut01', 'Salvatore', 'Cutugno', 'salvatore.cutugno@vsb.cz', 3);
INSERT INTO Teacher (login, fname, lname, email, department) VALUES ('cel01', 'Adriano', 'Celentano', 'adriano.celentano@vsb.cz', 4);

SELECT * FROM Department
SELECT * FROM Teacher

UPDATE Department SET head = 'luk06' WHERE id = 1;
UPDATE Department SET head = 'toz01' WHERE id = 2;
UPDATE Department SET head = 'cut01' WHERE id = 3;
UPDATE Department SET head = 'cel01' WHERE id = 4;

-- Slide 6/9 --------------------------------------------------------------------

BEGIN
	DECLARE @login CHAR(5);
	DECLARE @fname VARCHAR(30);
	DECLARE @lname VARCHAR(50);
	DECLARE @email VARCHAR(50);
	DECLARE @department INT;
	DECLARE @msg VARCHAR(1000);
	
	DECLARE c CURSOR FAST_FORWARD FOR	
		SELECT
			login, fname, lname, email, department
		FROM
			Teacher
		WHERE
			Department IN (
				SELECT department
				FROM Teacher
				GROUP BY department
				HAVING COUNT(*) > 1)
		ORDER BY
			Department
	OPEN c;
	
	FETCH NEXT FROM c INTO @login, @fname, @lname, @email, @department;
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @msg = @login + ': ' + @fname + ' ' + @lname + ', ' + @email + ', ' + CAST(@department AS VARCHAR);
		PRINT @msg;
		FETCH NEXT FROM c INTO @login, @fname, @lname, @email, @department;
	END;
	
	CLOSE c;
	DEALLOCATE c;
END;

-- Slide 6/9 --------------------------------------------------------------------
ALTER PROCEDURE spCopyTableSchema(@schema VARCHAR(50), @tableName VARCHAR(50), @newTableName VARCHAR(50) OUT) AS
BEGIN
	DECLARE @columnStatement NVARCHAR(200);
	DECLARE @sql NVARCHAR(MAX);
	DECLARE c CURSOR FAST_FORWARD FOR
	SELECT
		C.name +
		' ' + P.name +
		CASE WHEN P.name = 'varchar' OR P.name = 'char' THEN '(' + CAST(C.max_length AS VARCHAR) + ')' ELSE '' END +
		CASE WHEN C.is_nullable = 1 THEN ' NULL' ELSE ' NOT NULL' END
	FROM
		sys.tables T
		JOIN sys.schemas S ON T.schema_id = S.schema_id
		JOIN sys.columns C ON T.object_id = C.object_id
		JOIN sys.types P ON C.system_type_id = P.system_type_id
	WHERE
		T.name = @tableName AND S.name = @schema;
	OPEN c;
	
	SET @newTableName = @tableName + '_dcp';
	
	SET @sql = 'CREATE TABLE [' + @schema + '].[' + @newTableName + '] (' + CHAR(13) + CHAR(10);
	FETCH NEXT FROM c INTO @columnStatement;
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @sql += '    ' + @columnStatement;
		SET @sql += ',' + CHAR(13) + CHAR(10);
		FETCH NEXT FROM c INTO @columnStatement;
	END;
	CLOSE c;
	DEALLOCATE c;
	SET @sql += '    date_copy DATETIME NOT NULL)';
	
	EXEC sp_executesql @sql;
END;

ALTER PROCEDURE spCopyTableDate(@schema VARCHAR(50), @tableName VARCHAR(50)) AS
BEGIN
	DECLARE @newTableName NVARCHAR(MAX);
	EXEC spCopyTableSchema @schema, @tableName, @newTableName OUT;
	
	DECLARE @sql NVARCHAR(1000);
	SET @sql = 'INSERT INTO ' + @newTableName + ' SELECT *, GETDATE() FROM ' + @tableName;

	EXEC sp_executesql @sql;	
END;

EXEC spCopyTableDate 'dbo', 'Teacher';

-- Slide 6/9 --------------------------------------------------------------------

SELECT CAST('01-JAN-2009' AS DATETIME)
SELECT CAST('20090101' AS DATETIME)
SELECT CAST('2009-01-01' AS DATETIME)
SELECT CAST('2009/01/01' AS DATE)

SELECT CONVERT(DATE, '01/01/2009', 101);

SELECT GETDATE(), CURRENT_TIMESTAMP